/*
   RainbowCrack - a general propose implementation of Philippe Oechslin's
   faster time-memory trade-off technique.

   Copyright (C) Zhu Shuanglei <shuanglei@hotmail.com>
*/
#ifndef _GCHAINWALKCONTEXT_H
#define _GCHAINWALKCONTEXT_H

#ifdef _WIN32
	#include <windows.h>
#endif
#include <time.h>
#include "HashRoutine.h"
#include "Public.h"


class GCChainWalkContext {
 public:
  GCChainWalkContext(unsigned int nChainCounter,
		     unsigned int nRainbowChainCount,
		     unsigned int nRainbowChainLen);
  virtual ~GCChainWalkContext();

 public:
  // Should be private but using accessors lead to
  // a few % hit on performances.

  string m_sHashRoutineName;
  HASHROUTINE m_pHashRoutine;
  unsigned int m_nHashLen;

  unsigned char m_PlainCharset[256];
  unsigned int m_nPlainCharsetLen;
  unsigned int m_nPlainLenMin;
  unsigned int m_nPlainLenMax;
  string m_sPlainCharsetName;
  string m_sPlainCharsetContent;
  uint64 m_nPlainSpaceUpToX[MAX_PLAIN_LEN + 1];
  uint64 m_nPlainSpaceTotal;

  int m_nRainbowTableIndex;
  uint64 m_nReduceOffset;

  FILE *m_file;
  unsigned int m_nChainCounter;
  unsigned int m_nRainbowChainCount;
  unsigned int m_nRainbowChainLen;

  clock_t m_nStartClock;

#ifdef _WIN32
  HANDLE cmutex;
#else
  pthread_mutex_t cmutex;
#endif

 private:
  bool LoadCharset(string sName);

 public:
  bool SetHashRoutine(string sHashRoutineName);
  HASHROUTINE GetHashRoutine();
  bool SetPlainCharset(string sCharsetName,
		       int nPlainLenMin,
		       int nPlainLenMax);
  bool SetRainbowTableIndex(int nRainbowTableIndex);
  string GetHashRoutineName();
  unsigned int GetHashLen();
  unsigned char *GetPlainCharset();
  unsigned int GetPlainCharsetLen();
  string GetPlainCharsetName();
  string GetPlainCharsetContent();
  unsigned int GetPlainLenMin();
  unsigned int GetPlainLenMax();
  uint64 GetPlainSpaceTotal();
  uint64 *GetPlainSpaceUpToX();
  int GetRainbowTableIndex();
  uint64 GetReduceOffset();
  unsigned int GetRainbowChainLen();
  FILE *GetFile();
  void Dump();

  void SetFile(FILE *f);

  void PrintStats() const;
  bool IncCounterAndStop(unsigned int val);

  void AcquireLock();
  void ReleaseLock();
};

#endif
